/* Copyright 2004, Jan D. Wolter, All Rights Reserved. */

/* This file defines various data structures used in dumping/loading user
 * data.  It's not core backtalk stuff.
 */

#include "common.h"
#include "tag.h"
#include "str.h"
#include "vstr.h"
#include "partdir.h"
#include "btpasswd.h"
#include "group.h"
#include "dumputil.h"
#include <string.h>


/* Globals */
char **grplist;	/* An array of pointers to group names, indexed by group id */
int maxgrp;     /* The maximum group index */
int grpalloc= 0;/* The number of elements allocated for the group array */
struct plist *partlist;


/* PARTICIPATION FILE DIRECTORY - Given a user, return the full path of
 * the directory containing his .backtalk file and participation files.
 */

void pf_dir(char *bf, struct btpasswd *pw)
{
#ifdef PART_DIR
    bbspartpath(bf, pw->name, "");
#else
    homepartpath(bf, pw->dir, "");
#endif
}


/* RC FILE DIRECTORY - Given a user, return the full path of
 * the directory containing his .cfonce and .cfrc files.
 */

int rc_dir(char *bf, struct btpasswd *pw)
{
    homepartpath(bf, pw->dir, "");
}


/* ADD_GROUP - Add the group with the given name into the array structure
 * above.
 */

void add_group(char *grpname)
{
    int i;
    int grpid= groupid(grpname);

    if (grpid < 0) return;

    /* Allocate storage, if needed */
    if (grpid >= grpalloc)
    {
	if (grpalloc == 0)
	{
	    grpalloc= grpid + 10;
	    grplist= (char **)malloc(grpalloc*sizeof(char *));
	}
	else
	{
	    grpalloc= grpid + 20;
	    grplist= (char **)realloc(grplist,grpalloc*sizeof(char *));
	}
    }

    if (grpid > maxgrp)
    {
	/* Fill in any gap with NULL pointers */
	for (i= maxgrp + 1; i < grpid; i++)
	    grplist[i]= NULL;

	maxgrp= grpid;
    }

    /* Save the new group */
    grplist[grpid]= strdupo(grpname);
}


/* GET_USER_GROUPS - return a comma-separated list of groups that the user
 * is in, not including the the primary group passed in as an argument.
 * Value is returned in malloced memory.  Return NULL if the user is in no
 * other groups.
 */

char *get_user_groups(char *userid, int prime_gid)
{
    int i;
    VSTRING out;
    vs_new(&out, 0);

    for (i= 0; i < maxgrp; i++)
    {
	if (i != prime_gid &&
	    grplist[i] != NULL &&
	    ingroupno(i, prime_gid, userid))
	{
	    if (out.p > out.begin) *vs_inc(&out)= ',';
	    vs_cat(&out,grplist[i]);
	}
    }
    if (out.p > out.begin)
	return vs_return(&out);
    else
	return NULL;
}


/* READCFLIST - Load the conflist into a linked list of plist structures.
 * We skip any duplicate entries.  Conference name is taken as shortest
 * legal abbreviation that would not be shadowed by a previous abbreviation.
 * pf fields are not initialized.
 */

void readcflist()
{
    FILE *fp;
    char buf[BFSZ], *p, *q, *dir;
    struct plist *pl;
    int first= TRUE, clen;

    if ((fp= fopen(CONFLIST,"r")) == NULL)
	die("Could not open "CONFLIST);

    if (fgets(buf,BFSZ,fp) == NULL || strcmp(buf,"!<hl01>\n"))
	die("conference list "CONFLIST" has bad magic number");

    while (fgets(buf,BFSZ,fp) != NULL)
    {
	if (buf[0] == '#') continue;
	if (first) {first= FALSE; continue;}	/* skip first line */
	if ((p= firstin(buf,":")) == NULL)
	{
	    fclose(fp);
	    die("Bad line in "CONFLIST": '%s'", buf);
	}
	*(p++)= '\0';
	*(firstin(p,"\n"))= '\0';

	/* Find _ in name.  Set clen to minimum legal abbreviation */
	if ((q= strchr(buf,'_')) == NULL)
	    clen= strlen(buf);
	else
	{
	    clen= q-buf;
	    /* Remove _ from name */
	    for ( ; *q != '\0'; q++)
		q[0]= q[1];
	}

	/* Expand % in path name */
	if (p[0] == '%')
	{
	    dir= (char *)malloc(strlen(BBS_DIR)+strlen(p));
	    strcpy(dir,BBS_DIR);
	    strcat(dir,p+1);
	}
	else
	    dir= strdupo(p);

	/* Check if we already have entry with this directory name.
	 * While scanning, expand conference name abbreviation to avoid
	 * shadowing. */
	for (pl= partlist; pl != NULL && strcmp(pl->dr,dir); pl= pl->next)
	{
	    while (!strncmp(pl->cf, buf, clen))
	    {
		if (buf[clen] == '\0')
		{
		    fprintf(stderr,"Conference %s(%d) shadows %s(%s) in "
			CONFLIST"\n", pl->cf, pl->dr, buf, dir);
		    goto nextline;
		}
		clen++;
	    }
	}

	/* Add it in if not */
	if (pl == NULL)
	{
	    pl= (struct plist *)malloc(sizeof(struct plist));
	    pl->cf= strdupo(buf);
	    pl->cflen= clen;
	    pl->dr= dir;
	    pl->next= partlist;
	    partlist= pl;
	}

nextline:;
    }
    fclose(fp);
    if (first)
	die("conference list "CONFLIST" is too short");
}


/* MKPARTLIST - Get a list of all participation files with conference
 * names.  Goes into global partlist variable.
 */

void mkpartlist()
{
    struct plist *pl;
    char buf[BFSZ+1];
    FILE *fp;

    /* Get list from conflist file */
    partlist= NULL;
    readcflist();

    /* Look up participation file name for each.  This is just second line
     * of config file.
     */
    for (pl= partlist; pl != NULL; pl= pl->next)
    {
	sprintf(buf,"%s/config",pl->dr);
	if ((fp= fopen(buf,"r")) == NULL)
	    die("Cannot open conference config file %d",buf);
	if (fgets(buf,BFSZ,fp) == NULL || strcmp(buf,"!<pc02>\n"))
	    die("Bad config file version number: %s", buf);
	if (fgets(buf,BFSZ,fp) == NULL)
	    die("conference configuration file %s is missing name of "
		"participation file",buf);
	*firstin(buf,"\n")= '\0';
	pl->pf= strdupo(buf);
	fclose(fp);
    }
}


/* FINDPART - Find the partlist entry corresponding to a give participation
 * file name.
 */

struct plist *findpart(char *pfname)
{
    struct plist *pl;

    for (pl= partlist; pl != NULL; pl= pl->next)
	if (!strcmp(pl->pf,pfname))
	    return pl;
    return NULL;
}

